package game

import (
	"errors"
	"support-game/models"
	"support-game/models/game"
	"support-game/models/model"
	"support-game/utils"
	"time"
)

// SupportServ 活动serv
type SupportServ struct {
	ctx *utils.Context
	dao *game.SupportDao
}

// NewSupportServ 初始化
func NewSupportServ(ctx *utils.Context) *SupportServ {
	return &SupportServ{
		ctx: ctx,
		dao: game.NewSupportDao(ctx),
	}
}

// GetCustomerByCode 获取用户信息
func (s *SupportServ) GetCustomerByCode(code string) (*model.TaCustomer, error) {
	if code == "" {
		return nil, errors.New("code为空")
	}
	data, err := utils.WxClient().GetUserInfo(code)
	if err != nil {
		utils.LogError("获取微信用户信息失败：", err)
		return nil, err
	}

	openid := data["openid"].(string)
	customer, err := s.dao.GetCustomerByOpenID(openid)
	if err != nil {
		utils.LogError("获取用户信息失败：", err)
		return nil, err
	}

	if customer != nil && customer.CustomerId != "" && customer.CustomerName != "" {
		return customer, nil
	}

	user := model.TaCustomer{}
	user.Openid = openid

	if data["sex"] != nil {
		user.Sex = int(data["sex"].(float64))
	}

	if data["nickname"] != nil {
		user.CustomerName = data["nickname"].(string)
	}

	if data["headimgurl"] != nil {
		user.HeadImg = data["headimgurl"].(string)
	}

	if customer != nil && customer.CustomerId != "" {
		user.CustomerId = customer.CustomerId
		err = s.dao.UpdateCustomer(&user, []string{
			"sex",
			"customer_name",
			"head_img",
		})

		if err != nil {
			utils.LogError("修改用户信息失败", err)
			return nil, err
		}
		return &user, nil
	}

	saveUser, err := s.dao.SaveCustomer(user)
	if err != nil {
		utils.LogError("保存用户信息失败", err)
		return nil, err
	}
	return saveUser, nil
}

// SignUp 报名
func (s *SupportServ) SignUp(customerid, name, phone, address string) (*model.TaCustomer, error) {
	if customerid == "" {
		return nil, errors.New("没有需要报名的用户信息")
	}
	customer, err := s.dao.GetCustomerByID(customerid)
	if err != nil {
		utils.LogError("获取用户信息失败", err)
		return nil, err
	}
	customer.Phone = phone
	customer.Name = name
	customer.Address = address
	err = s.dao.UpdateCustomer(customer, []string{
		"name",
		"phone",
		"address",
	})
	if err != nil {
		utils.LogError("报名失败：", err)
		return nil, err
	}
	return customer, nil
}

// AddRecord 新增活动
func (s *SupportServ) AddRecord(customerid, gameid string) (*model.TaGameRecord, error) {
	game, err := s.dao.GetGameByID(gameid)
	if err != nil {
		utils.LogError("获取主信息信息失败", err)
		return nil, err
	}
	if game.Status == models.STATUS_INVALID {
		return nil, errors.New("活动已结束")
	}
	// 时间判断
	if time.Now().Before(game.BeginDate) {
		return nil, errors.New("活动未开始")
	}
	if time.Now().After(game.EndDate) {
		return nil, errors.New("活动已结束")
	}
	todayNote, err := s.dao.GetGameNote(gameid)
	if err != nil {
		utils.LogError("获取活动设置信息失败：", err)
		return nil, err
	}
	if todayNote == nil || todayNote.NotesId == "" {
		utils.LogError("获取当天所有成功数据失败：", err)
		return nil, errors.New("获取活动信息失败")
	}
	if todayNote.LeftNum <= 0 {
		return nil, errors.New("今日活动已结束！请明日再来！")
	}
	// 判断当天是否已参与过活动
	records, err := s.dao.GetTodayCustomerRecord(customerid, gameid)
	if err != nil {
		utils.LogError("判断当天是否参与过活动失败：", err)
		return nil, err
	}
	if len(records) > 0 {
		return nil, errors.New("您今天已参与过活动了，请明天再来吧！")
	}
	// 判断用户是否已成功
	records, err = s.dao.GetCustomerQuota(customerid, gameid)
	if err != nil {
		utils.LogError("判断当天是否参与过活动失败：", err)
		return nil, err
	}
	if len(records) > 0 {
		return nil, errors.New("您已参加此次活动，请勿重复参加！")
	}

	var record = model.TaGameRecord{
		CustomerId: customerid,
		GameId:     gameid,
		GameName:   game.GameName,
	}

	info, err := s.dao.AddRecord(record)
	if err != nil {
		utils.LogError("新增报名信息失败：", err)
		return nil, err
	}

	return info, nil
}

// AddSupport 助力
func (s *SupportServ) AddSupport(recordid, customerid string) (map[string]interface{}, error) {
	// 判断活动是否已结束
	record, err := s.dao.GetRecordByID(recordid)
	if record.Status == models.STATUS_INVALID {
		return nil, errors.New("当前活动已结束")
	}
	if record.Status == models.STATUS_NORMAL {
		return nil, errors.New("当前活动已助力成功")
	}
	game, err := s.dao.GetGameByID(record.GameId)
	if err != nil {
		utils.LogError("获取活动信息失败：", err)
		return nil, err
	}
	// 时间判断
	if time.Now().Before(game.BeginDate) {
		return nil, errors.New("活动未开始")
	}
	if time.Now().After(game.EndDate) {
		return nil, errors.New("活动已结束")
	}
	// 判断用户信息
	cust, err := s.dao.GetCustomerByID(customerid)
	if err != nil {
		utils.LogError("获取活动信息失败：", err)
		return nil, err
	}
	if cust == nil || cust.CustomerId == "" {
		return nil, errors.New("当前用户状态不正确！")
	}
	if cust.CustomerId == record.CustomerId {
		return nil, errors.New("您不能给自己助力！")
	}
	// 判断是否助力过
	support, err := s.dao.GetCustomerSupport(recordid, customerid)
	if err != nil {
		utils.LogError("获取用户助力信息失败：", err)
		return nil, err
	}
	if len(support) > 0 {
		return nil, errors.New("您已助力过此活动！")
	}
	// 助力
	var addinfo = model.TaRecordSuppot{
		RecordId:   recordid,
		CustomerId: customerid,
	}
	info, err := s.dao.AddSupport(addinfo)
	if err != nil {
		utils.LogError("新增助力信息失败：", err)
		return nil, err
	}

	supports, err := s.dao.GetSupportsByRecord(recordid)
	if err != nil {
		utils.LogError("获取助力信息失败：", err)
		return nil, err
	}
	if len(supports) >= game.UserNum {
		// 助力成功
		record.Status = models.STATUS_NORMAL
		err := s.dao.UpdateRecord(record, []string{
			"status",
		})
		if err != nil {
			utils.LogError("更新助力信息失败：", err)
			return nil, err
		}
		// 更新当天的剩余次数
		row, err := s.dao.UpdateLeftNum(record.GameId)
		if err != nil {
			utils.LogError("更新剩余天数失败：", err)
			return nil, err
		}
		if row <= 0 {
			return nil, errors.New("活动已结束")
		}
	}
	return map[string]interface{}{
		"record":  record,
		"support": info,
	}, nil
}

// GetRecordInfo 获取活动信息
func (s *SupportServ) GetRecordInfo(recordid string) (map[string]interface{}, error) {
	record, err := s.dao.GetRecordByID(recordid)
	if err != nil {
		utils.LogError("获取活动信息失败：", err)
		return nil, err
	}
	supports, err := s.dao.GetSupportsByRecord(recordid)
	if err != nil {
		utils.LogError("获取助力信息失败：", err)
		return nil, err
	}
	customer, err := s.dao.GetCustomerByID(record.CustomerId)
	return map[string]interface{}{
		"record":   record,
		"supports": supports,
		"customer": customer,
	}, nil
}

// GetGameInfo 获取活动主信息
func (s *SupportServ) GetGameInfo(gameid, customerid string) (map[string]interface{}, error) {
	game, err := s.dao.GetGameByID(gameid)
	if err != nil {
		utils.LogError("获取主信息失败：", err)
		return nil, err
	}
	note, err := s.dao.GetGameNote(gameid)
	if err != nil {
		utils.LogError("获取进入抽奖信息失败：", err)
		return nil, err
	}
	// 获取用户的抽奖信息
	records, err := s.dao.GetCustomerQuota(customerid, gameid)
	if err != nil {
		utils.LogError("获取用户抽奖信息失败：", err)
		return nil, err
	}
	var nowRecord model.TaGameRecord
	if len(records) > 0 {
		nowRecord = records[0]
	} else {
		// 获取用户当天的抽奖信息
		records, err = s.dao.GetTodayCustomerRecord(customerid, gameid)
		if err != nil {
			utils.LogError("获取用户当天抽奖信息失败：", err)
			return nil, err
		}
		if len(records) > 0 {
			nowRecord = records[0]
		}
	}

	return map[string]interface{}{
		"game":   game,
		"note":   note,
		"record": nowRecord,
	}, nil
}
